#!/usr/bin/python
# *************************************************************************************************
#
# Horde3D
#   Next-Generation Graphics Engine
#
# Sample Application
# --------------------------------------
# Copyright (C) 2006-2008 Nicolas Schulz
#               2008 Florian Noeding
#
#
# This sample source file is not covered by the LGPL as the rest of the SDK
# and may be used without any restrictions
#
# *************************************************************************************************

import pyglet
import pyglet.gl
import pyglet.window
import pyglet.clock

import horde3d as h3d

from optparse import OptionParser

class App(object):
	def __init__(self):
		pass

	def init(self):
		self._parseOptions()

		self._initGL()
		self._initHorde3D()

	def _parseOptions(self):
		op = OptionParser()
		op.add_option('-x', '--width', type='int', dest='width', help='window width', default=800)
		op.add_option('-y', '--height', type='int', dest='height', help='window height', default=600)
		op.add_option('--vsync-on', dest='vsync', help='enable / disable vsync', action='store_true', default=False)
		op.add_option('--content', dest='contentPaths', help='path to content directory, may be specified several times', action='append', default=[])
		op.add_option('--limit-fps', type='int', dest='fpsLimit', help='limit FPS to specified value; may improve keyboard / mouse reaction times for values below 100', default=None)

		self._parseOptions_moreOptions(op)

		options, args = op.parse_args()
		self._options = options
		self._args = args

		self._width = options.width; options.width = None
		self._height = options.height; options.height = None

		if not options.contentPaths:
			op.error('no content path set')

		self._parseOptions_moreChecks(options, args)

	def _parseOptions_moreOptions(self, op):
		# override to add more options using op.add_action(...)
		pass

	def _parseOptions_moreChecks(self, options, args):
		# override to check more options, on error call op.error('message')
		pass


	def _initGL(self):
		# create window
		#   32 bit ARGB
		#   24 bit depth buffer
		#   double buffering
		config = pyglet.gl.Config(buffer_size=32, depth_size=24, double_buffer=True)
		self._window = pyglet.window.Window(
				resizable=True,
				config=config,
				width=self._width,
				height=self._height,
				caption='Horde3D - Python wrapper test',
				vsync=self._options.vsync)

		# event dispatch
		self._window.on_resize = self._onResize
		self._window.on_key_press = self._onKeyPress
		self._window.on_key_release = self._onKeyRelease
		self.keyboard = pyglet.window.key.KeyStateHandler()
		self._window.push_handlers(self.keyboard)

		self._window.on_mouse_motion = self._onMouseMotion
		self._window.on_mouse_press = self._onMousePress
		self._window.on_mouse_release = self._onMouseRelease
		self._window.on_mouse_drag = self._onMouseDrag
		self._window.on_mouse_scroll = self._onMouseScroll

		# FPS limit
		pyglet.clock.set_fps_limit(self._options.fpsLimit)


	def _initHorde3D(self):
		# init Horde3D
		h3d.init()

		self._h3dResourceLocations()
		self._h3dEngineOptions()
		self._h3dAddResources()

		# load resources from disk
		if not h3d.utils.loadResourcesFromDisk('|'.join(self._options.contentPaths)):
			print 'loading of some resources failed: See EngineLog.html'

		self._h3dSetupCamera()

		self._h3dSetupScene()



	def _h3dResourceLocations(self):
		# resource locations
		h3d.utils.setResourcePath(h3d.ResourceTypes.SceneGraph, 'models')
		h3d.utils.setResourcePath(h3d.ResourceTypes.Geometry, 'models')
		h3d.utils.setResourcePath(h3d.ResourceTypes.Animation, 'models')
		h3d.utils.setResourcePath(h3d.ResourceTypes.Material, 'materials')
		h3d.utils.setResourcePath(h3d.ResourceTypes.Code, 'shaders')
		h3d.utils.setResourcePath(h3d.ResourceTypes.Shader, 'shaders')
		h3d.utils.setResourcePath(h3d.ResourceTypes.Texture2D, 'textures')
		h3d.utils.setResourcePath(h3d.ResourceTypes.TextureCube, 'textures')
		h3d.utils.setResourcePath(h3d.ResourceTypes.Effect, 'effects')
		h3d.utils.setResourcePath(h3d.ResourceTypes.Pipeline, 'pipelines')

	def _h3dEngineOptions(self):
		# engine options
		h3d.setOption(h3d.EngineOptions.LoadTextures, 1)
		h3d.setOption(h3d.EngineOptions.TexCompression, 0)
		h3d.setOption(h3d.EngineOptions.FastAnimation, 0)
		h3d.setOption(h3d.EngineOptions.AnisotropyFactor, 8)
		h3d.setOption(h3d.EngineOptions.ShadowMapSize, 2048)

	def _h3dAddResources(self):
		# add resources
		class H3DRes:
			pass
		h3dres = H3DRes()
		self._h3dres = h3dres

		h3dres.forwardPipe = h3d.addResource(h3d.ResourceTypes.Pipeline, 'forward.pipeline.xml', 0)
		h3dres.fontMaterial = h3d.addResource(h3d.ResourceTypes.Material, 'font.material.xml', 0)
		h3dres.logoMaterial = h3d.addResource(h3d.ResourceTypes.Material, 'logo.material.xml', 0)



	def _h3dSetupCamera(self):
		# add camera
		self._camera = h3d.addCameraNode(h3d.RootNode, 'Camera', self._h3dres.forwardPipe)
		h3d.setNodeTransform(self._camera, 0, 0, -100, 0, 0, 0, 1, 1, 1)


	def _h3dSetupScene(self):
		pass

		
	
	def _onResize(self, width, height):
		self._width = width
		self._height = height

		h3d.resize(0, 0, width, height)

		h3d.setupCameraView(self._camera, 45, width / float(height), 0.1, 1000)


	def _onKeyPress(self, symbol, modifiers):
		key = pyglet.window.key

		if symbol == key.ESCAPE:
			self._window.has_exit = True


	def _onKeyRelease(self, symbol, modifiers):
		pass


	def _onMouseMotion(self, x, y, dx, dy):
		pass

	def _onMousePress(self, x, y, button, modifiers):
		pass

	def _onMouseRelease(self, x, y, button, modifiers):
		pass

	def _onMouseDrag(self, x, y, dx, dy, buttons, modifiers):
		pass

	def _onMouseScroll(self, x, y, scrollX, scrollY):
		pass

	def _mainloopUpdate(self, dt):
		pass


	def mainloop(self):
		w = self._window
		fpsTimer = 0.0
		fps = 0.0
		fpsString = ''
		while not w.has_exit:
			# dispatch window messages
			w.dispatch_events()

			dt = pyglet.clock.tick()
			fpsTimer += dt
			if fpsTimer > 0.3:
				fpsTimer = 0.0
				fps = pyglet.clock.get_fps()
				fpsString = ('%.2f' % fps).rjust(8, ' ')

			self._mainloopUpdate(dt)

			h3d.utils.showText(fpsString, 0, 0.95, 0.03, 0, self._h3dres.fontMaterial)


			h3d.showOverlay(0.75, 0, 0, 0, 1, 0, 1, 0, 1, 0.2, 1, 1, 0.75, 0.2, 0, 1, 7, self._h3dres.logoMaterial)
			h3d.render(self._camera)

			h3d.clearOverlays()
			h3d.utils.dumpMessages()


			w.flip()




